package com.lvwan.mobile110.activity

import android.app.Activity
import android.content.Intent
import android.media.MediaScannerConnection
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import com.common.activity.BindingActivity
import com.common.annotation.LayoutId
import com.common.task.MultiTaskManager
import com.common.task.Task
import com.lvwan.mobile110.R
import com.lvwan.mobile110.databinding.ActivityFaceAppealBinding
import com.lvwan.mobile110.entity.event.AppealEvent
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.mobile110.viewmodel.FaceAppealViewModel
import com.lvwan.util.ImageUtils
import com.lvwan.util.ToastUtils
import com.nineoldandroids.animation.Animator
import com.nineoldandroids.view.ViewPropertyAnimator
import kotlinx.android.synthetic.main.activity_face_appeal.*
import org.greenrobot.eventbus.EventBus


/**
 * Created by wangdong on 16/12/16.
 * copy from UserIdCardFrontActivity
 */

@LayoutId(R.layout.activity_face_appeal)
class FaceAppealActivity : BindingActivity<FaceAppealViewModel, ActivityFaceAppealBinding>() {


    private var mFrontFilePath = ""

    fun onSelectImg() {
        UserIdCardChooseImageActivity.startForResult(this, REQUEST_CODE_IMAGE)
    }

    fun onSubmit() {
        loading.visibility = View.VISIBLE
        val task = object : Task<Any, Any>() {
            override fun run(params: Any): Any? {
                mFrontFilePath = ImageUtils.compressImage(mFrontFilePath)
                RequestManager.instance().postIdcardAppeal(viewModel.idcard.get(), viewModel.name.get(),
                        viewModel.phone.get(), mFrontFilePath
                ) { lwBean ->
                    if (lwBean.existError()) {
                        ToastUtils.getInstance().showToast(lwBean.getMessage())
                    } else {
                        EventBus.getDefault().post(AppealEvent())
                        this@FaceAppealActivity.finish()
                    }
                    loading.visibility = View.GONE
                }
                return null
            }
        }
        MultiTaskManager.execute(task)

    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        select_img.setOnClickListener { onSelectImg() }
        help.setOnClickListener { showHelpView() }
        help_view.setOnClickListener { hideHelpView() }
        btn_ok.setOnClickListener { onSubmit() }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_CODE_IMAGE && Activity.RESULT_OK == resultCode && data != null) {
            mFrontFilePath = data.getStringExtra(UserIdCardChooseImageActivity.KEY_DATA)
            MediaScannerConnection.scanFile(this, arrayOf(mFrontFilePath), null, null)
            ImageUtils.displayImage(mFrontFilePath, image)
            select_img.setBackgroundResource(R.drawable.user_id_card_front_img)
            viewModel.checkEnable()
        }
    }

    private fun showHelpView() {
        help_view.visibility = View.VISIBLE
        help_view.scaleX = 0f
        help_view.scaleY = 0f
        val animator = ViewPropertyAnimator.animate(help_view).scaleX(1f).scaleY(1f).setDuration(300)
        animator.setListener(null)
        animator.start()
    }

    private fun hideHelpView() {

        val pos = IntArray(2)
        help.getLocationInWindow(pos)
        val x = pos[0] + help.width / 2
        val y = pos[1]

        help_view.pivotX = x.toFloat()
        help_view.pivotY = y.toFloat()

        val animator = ViewPropertyAnimator.animate(help_view).scaleX(0f).scaleY(0f).setDuration(300)
        animator.setListener(object : Animator.AnimatorListener {
            override fun onAnimationStart(animator: Animator) {

            }

            override fun onAnimationEnd(animator: Animator) {
                help_view.visibility = View.GONE
            }

            override fun onAnimationCancel(animator: Animator) {
                help_view.visibility = View.GONE
            }

            override fun onAnimationRepeat(animator: Animator) {

            }
        })
        animator.start()
    }

    fun hasImage(): Boolean {
        return !TextUtils.isEmpty(mFrontFilePath)
    }

    companion object {

        private val REQUEST_CODE_IMAGE = 110
    }
}
