package com.lvwan.mobile110.activity

import android.app.Fragment
import android.os.Bundle
import android.text.TextUtils
import com.common.activity.BindingActivity
import com.common.annotation.LayoutId
import com.jakewharton.rxbinding.widget.RxTextView
import com.lvwan.mobile110.R
import com.lvwan.mobile110.databinding.ActivityGovguideBinding
import com.lvwan.mobile110.entity.event.GuideQueryEvent
import com.lvwan.mobile110.fragment.*
import com.lvwan.mobile110.stat.Action
import com.lvwan.mobile110.stat.ActionId
import com.lvwan.mobile110.stat.Module
import com.lvwan.mobile110.stat.ModuleId
import com.lvwan.mobile110.viewmodel.GovGuideViewModel
import com.lvwan.util.LocationUtil
import org.greenrobot.eventbus.Subscribe
import rx.android.schedulers.AndroidSchedulers
import rx.functions.Func1
import java.util.concurrent.TimeUnit

/**
 *
 * Created by wangdong on 16/12/4.
 */

@ModuleId(Module.BSZN)
@ActionId(Action.BSZN)
@LayoutId(R.layout.activity_govguide)
class GovGuideActivty : BindingActivity<GovGuideViewModel, ActivityGovguideBinding>() {

    private val fragments = arrayOfNulls<Fragment>(5)
    private val fragmentTags = arrayOfNulls<String>(5)

    private var page = -1

    var city: String? = null
    private var cond: String? = null
    private var target: String? = null
    private var workTheme: String? = null
    private var department: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        initFragments()
        monitorSearch()
        setDefault()
        setPage(PAGE_RESULT)
        setPage(PAGE_BUSINESS)
    }

    private fun setDefault() {
        try {
            val location = LocationUtil.getInstance().location
            city = location.city
            viewModel.setTitle0(city)
        } catch (ignored: NullPointerException) {
        }

    }


    private fun initFragments() {
        fragments[PAGE_BUSINESS] = GuideBusinessFragment()
        fragments[PAGE_LOCATION] = LocationFragment()
        fragments[PAGE_DEPARTMENT] = GuideDepartmentFragment()
        fragments[PAGE_MORE] = GuideMoreFragment()
        fragments[PAGE_RESULT] = GuideResultFragment()
        fragmentTags[PAGE_BUSINESS] = GuideBusinessFragment.TAG
        fragmentTags[PAGE_LOCATION] = LocationFragment.TAG
        fragmentTags[PAGE_DEPARTMENT] = GuideDepartmentFragment.TAG
        fragmentTags[PAGE_MORE] = GuideMoreFragment.TAG
        fragmentTags[PAGE_RESULT] = GuideResultFragment.TAG
    }

    fun setPage(page: Int) {
        this.page = page
        var tag: String? = null
        when (page) {
            PAGE_BUSINESS -> tag = GuideBusinessFragment.TAG

            PAGE_LOCATION -> tag = LocationFragment.TAG

            PAGE_DEPARTMENT -> {
                val fragment = fragments[PAGE_DEPARTMENT] as GuideDepartmentFragment
                fragment.setCity(city)
                tag = GuideDepartmentFragment.TAG
            }

            PAGE_MORE -> tag = GuideMoreFragment.TAG

            PAGE_RESULT -> tag = GuideResultFragment.TAG
        }

        val ft = fragmentManager.beginTransaction()
        for (i in fragmentTags.indices) {
            if (fragmentManager.findFragmentByTag(fragmentTags[i]) != null) {
                ft!!.hide(fragments[i])
            }
        }
        ft?.commitAllowingStateLoss()

        val fragment = fragments[page]

        if (fragmentManager.findFragmentByTag(tag) != null) {
            fragmentManager.beginTransaction().show(fragment).commitAllowingStateLoss()
        } else {
            fragmentManager.beginTransaction().add(R.id.fragment_container, fragment, tag).commitAllowingStateLoss()
        }

    }


    @Subscribe
    fun onQuery(event: GuideQueryEvent) {
        when (event.type) {
            GuideQueryEvent.TYPE_AREA -> {
                city = event.area
                viewModel.setTitle0(city)
            }
            GuideQueryEvent.TYPE_COND -> {
                viewModel.selectNull()
                cond = event.cond
            }
            GuideQueryEvent.TYPE_DEPARTMENT -> {
                department = event.department
                viewModel.setTitle1(department)
                if (event.invalid) {
                    department = null
                }
            }
            GuideQueryEvent.TYPE_TARGET -> {
                target = event.target
                viewModel.setTitle2_1(target)
                if (event.invalid) {
                    target = null
                }
            }
            GuideQueryEvent.TYPE_WORKTHEME -> {
                workTheme = event.workTheme
                viewModel.setTitle2_2(workTheme)
            }
        }
        if (TextUtils.isEmpty(city) && TextUtils.isEmpty(cond) && TextUtils.isEmpty(department) && TextUtils.isEmpty(target) && TextUtils.isEmpty(workTheme)) {
            setPage(PAGE_BUSINESS)
            viewModel.selectNull()
        } else {
            setPage(PAGE_RESULT)
            val fragment = fragments[PAGE_RESULT] as GuideResultFragment
            if (fragment.isAdded) {
                fragment.setParams(city, cond, department, target, workTheme)
                if (!fragment.isHidden) {
                    fragment.requestList()
                }
            }
        }

    }

    private fun monitorSearch() {
        RxTextView.textChanges(binding.guideSearch)
                .filter { page >= 0 }
                .debounce(200, TimeUnit.MILLISECONDS)
                .map<String>({ it.toString() })
                .filter { s -> s != cond }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe { s ->
                    val event = GuideQueryEvent(GuideQueryEvent.TYPE_COND)
                    event.cond = s
                    onQuery(event)
                }
    }

    companion object {

        val PAGE_BUSINESS = 0
        val PAGE_LOCATION = 1
        val PAGE_DEPARTMENT = 2
        val PAGE_MORE = 3
        val PAGE_RESULT = 4
    }


}
