package com.lvwan.mobile110.activity

import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import com.baidu.mapapi.model.LatLng
import com.baidu.mapapi.search.core.SearchResult
import com.baidu.mapapi.search.poi.*
import com.common.activity.BindingActivity
import com.common.annotation.LayoutId
import com.jakewharton.rxbinding.widget.RxTextView
import com.lvwan.mobile110.R
import com.lvwan.mobile110.constant.CBundle
import com.lvwan.mobile110.databinding.ActivityGuardAddrBinding
import com.lvwan.mobile110.entity.event.GuardSetEvent
import com.lvwan.mobile110.entity.realm.GuardAddrRealm
import com.lvwan.mobile110.viewmodel.GuardAddrViewModel
import com.lvwan.util.LocationUtil
import kotlinx.android.synthetic.main.activity_guard_addr.*
import org.greenrobot.eventbus.Subscribe
import rx.android.schedulers.AndroidSchedulers
import java.util.concurrent.TimeUnit

@LayoutId(R.layout.activity_guard_addr)
class GuardAddrActivity : BindingActivity<GuardAddrViewModel, ActivityGuardAddrBinding>() {


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        window.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        monitorSearch()
    }

    override fun createViewModel(): GuardAddrViewModel {
        val tripStart = intent.getBooleanExtra(CBundle.TRIP_START, false)
        return GuardAddrViewModel(this, tripStart)
    }

    private fun monitorSearch() {
        RxTextView.textChanges(guard_addr_search)
                .debounce(100, TimeUnit.MILLISECONDS)
                .map<String>({ it.trim().toString() })
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe { s -> search(s) }
    }

    private fun search(text : String) {
        if (text.isEmpty()) {
            viewModel.refreshItems(null, true)
        } else {
            val poiSearch = PoiSearch.newInstance()
            poiSearch.setOnGetPoiSearchResultListener(searchResultListener)
            val latlng = getLatLng()
            if (latlng != null) {
                poiSearch.searchNearby(PoiNearbySearchOption().keyword(text).location(latlng).pageNum(0))
            }
        }
    }

    private fun getLatLng(): LatLng? {
        val location = LocationUtil.getInstance().location
        var latlng: LatLng? = null
        if (location != null) {
            latlng = LatLng(location.latitude, location.longitude)
        }
        return latlng
    }



    private val searchResultListener = object : OnGetPoiSearchResultListener {

        override fun onGetPoiResult(poiResult: PoiResult) {
            if (poiResult.error == SearchResult.ERRORNO.NO_ERROR) {
                if (poiResult.allPoi != null) {
                    val addrs = mutableListOf<GuardAddrRealm>()
                    for (poi in poiResult.allPoi) {
                        val addrRealm = GuardAddrRealm()
                        addrRealm.name = poi.name
                        addrRealm.addr = poi.address
                        addrRealm.latitude = poi.location.latitude
                        addrRealm.longitude = poi.location.longitude
                        addrs.add(addrRealm)
                    }
                    viewModel.refreshItems(addrs, false)
                } else {
                    viewModel.refreshItems(null, false)
                }

            }


        }

        override fun onGetPoiDetailResult(poiDetailResult: PoiDetailResult) {

        }

    }

    @Subscribe
    fun onSetAddr(event : GuardSetEvent) {
        viewModel.setAddr(event.home)
    }


}
