package com.lvwan.mobile110.activity

import android.app.Fragment
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.text.TextUtils

import com.common.activity.BindingActivity
import com.common.annotation.LayoutId
import com.lvwan.mobile110.R
import com.lvwan.mobile110.constant.CBundle
import com.lvwan.mobile110.constant.CPreference
import com.lvwan.mobile110.databinding.ActivityMotBinding
import com.lvwan.mobile110.entity.event.MOTCountyEvent
import com.lvwan.mobile110.fragment.MOTAddressListFragment
import com.lvwan.mobile110.fragment.MOTLocationFragment
import com.lvwan.mobile110.stat.Action
import com.lvwan.mobile110.stat.ActionId
import com.lvwan.mobile110.stat.Module
import com.lvwan.mobile110.stat.ModuleId
import com.lvwan.mobile110.viewmodel.MOTViewModel
import com.lvwan.util.PreferenceHelper
import kotlinx.android.synthetic.main.activity_mot.*

import org.greenrobot.eventbus.Subscribe
import org.greenrobot.eventbus.ThreadMode
import org.json.JSONException
import org.json.JSONObject


/**
 * 车辆年检
 *
 * Created by wangdong on 16/8/1.
 */

@ModuleId(Module.CLNJ)
@ActionId(Action.CLNJ)
@LayoutId(R.layout.activity_mot)
class MOTActivity : BindingActivity<MOTViewModel, ActivityMotBinding>() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        initFragments()
        mot_location.setOnClickListener { onSwitchFragment() }
    }


    private fun initFragments() {
        val json = PreferenceHelper.getString(CPreference.MOT_DETAIL_CITY_JSON)
        if (TextUtils.isEmpty(json)) {
            // no cache
            showLocationFragment(null)
            viewModel.setArrowSelected(false)
        } else {
            // have cache
            showAddressListFragment(json)
            viewModel.setArrowSelected(true)
        }
    }

    fun onSwitchFragment() {
        val json = PreferenceHelper.getString(CPreference.MOT_DETAIL_CITY_JSON)
        if (fragmentManager.findFragmentByTag(MOTLocationFragment::class.java.simpleName) != null) {
            if (!TextUtils.isEmpty(json)) {
                showAddressListFragment(json)
            }
            viewModel.setArrowSelected(true)

        } else {
            val bundle = Bundle()
            bundle.putString(CBundle.MOT_DETAIL_CITY_JSON, json)
            showLocationFragment(bundle)
            viewModel.setArrowSelected(false)
        }
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    fun onCountyEvent(event: MOTCountyEvent) {
        if (event.isClick) {
            showAddressListFragment(event.provinceId, event.cityId, event.countyId, event.city, event.county)
        } else {
            setLocation(event.city, event.countyId, event.county)
        }
    }

    private fun showAddressListFragment(provinceId: String, cityId: String, countyId: String, city: String, county: String) {
        val bundle = Bundle()
        bundle.putString(CBundle.PRIVANCE_ID, provinceId)
        bundle.putString(CBundle.CITY_ID, cityId)
        bundle.putString(CBundle.COUNTY_ID, countyId)
        bundle.putString(CBundle.CITY, city)
        bundle.putString(CBundle.COUNTY, county)
        showAddressListFragment(bundle)
        setLocation(city, countyId, county)
    }

    private fun setLocation(city: String, countyId: String, county: String) {
        viewModel.setLocation(city, if ("0" == countyId) "" else county)
    }

    private fun showAddressListFragment(json: String) {
        try {
            val jsonObj = JSONObject(json)
            val privinceId = jsonObj.getString(CBundle.PRIVANCE_ID)
            val cityId = jsonObj.getString(CBundle.CITY_ID)
            val countyId = jsonObj.getString(CBundle.COUNTY_ID)
            val city = jsonObj.getString(CBundle.CITY)
            val county = jsonObj.getString(CBundle.COUNTY)
            showAddressListFragment(privinceId, cityId, countyId, city, county)
        } catch (e: JSONException) {
            e.printStackTrace()
        }

    }

    private fun showAddressListFragment(args: Bundle) {
        fragmentManager.popBackStackImmediate()
        startFragment(MOTAddressListFragment::class.java, args)
    }

    private fun showLocationFragment(args: Bundle?) {
        fragmentManager.popBackStackImmediate()
        startFragment(MOTLocationFragment::class.java, args)
    }

    private fun startFragment(clazz: Class<*>, args: Bundle?) {
        val tag = clazz.simpleName
        val fragment = Fragment.instantiate(this, clazz.name, args)
        fragmentManager
                .beginTransaction()
                .replace(R.id.fragment_container, fragment, tag)
                .commitAllowingStateLoss()
    }

    companion object {


        fun start(context: Context) {
            val intent = Intent(context, MOTActivity::class.java)
            context.startActivity(intent)
        }
    }
}
