package com.lvwan.mobile110.fragment

import android.os.Bundle
import com.baidu.location.BDLocation
import com.baidu.mapapi.map.*
import com.baidu.mapapi.model.LatLng
import com.common.annotation.LayoutId
import com.common.fragment.BindingFragment
import com.lvwan.mobile110.R
import com.lvwan.mobile110.activity.GuardAddrActivity
import com.lvwan.mobile110.constant.CBundle
import com.lvwan.mobile110.databinding.FragmentGuardTripBinding
import com.lvwan.mobile110.entity.event.GuardChooseEvent
import com.lvwan.mobile110.viewmodel.GuardTripViewModel
import com.lvwan.util.LocationUtil
import com.lvwan.util.LocationUtil.LocationType
import kotlinx.android.synthetic.main.fragment_guard_trip.*
import org.greenrobot.eventbus.Subscribe
import org.jetbrains.anko.onClick
import org.jetbrains.anko.startActivity

/**
 *
 * Created by wangdong on 17/3/2.
 */

@LayoutId(R.layout.fragment_guard_trip)
class GuardTripFragment : BindingFragment<GuardTripViewModel, FragmentGuardTripBinding>() {

    val startDescriptor: BitmapDescriptor
    val destDescriptor: BitmapDescriptor

    init {
        startDescriptor = BitmapDescriptorFactory.fromResource(R.drawable.guard_loc_start)
        destDescriptor = BitmapDescriptorFactory.fromResource(R.drawable.guard_loc_end)
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)

        // mapview config
        guard_map.showZoomControls(false)
        guard_map.showScaleControl(false)

        // locate
        requestLocation()

        initViewActions()
    }

    private fun initViewActions() {
        guard_location.onClick { requestLocation() }
        guard_trip_start.onClick { startActivity<GuardAddrActivity>(CBundle.TRIP_START to true) }
        guard_trip_end.onClick { startActivity<GuardAddrActivity>(CBundle.TRIP_START to false) }
    }

    fun requestLocation() {
        registerLocation(true)
        LocationUtil.getInstance().requestLocation()
    }

    fun registerLocation(register: Boolean) {
        if (register) {
            LocationUtil.getInstance().registerLocationListener(locationListener)
        } else {
            LocationUtil.getInstance().unRegisterLocationListener(locationListener)
        }
    }

    val locationListener = { locationType: LocationType, bdLocation: BDLocation? ->
        if (locationType == LocationType.Succeed) {
            onLocation(bdLocation)
        }
        registerLocation(false)
    }

    private fun onLocation(location: BDLocation?) {
        activity?.let { location?.let  {
            // fill descriptor
            fillDescriptor(it.latitude, it.longitude, true)
        } }
    }

    private fun fillDescriptor(latitude: Double, longitude: Double, start: Boolean) {

        // animate
        val latLng = LatLng(latitude, longitude)
        val u = MapStatusUpdateFactory.newLatLngZoom(latLng, 17.0f)
        map()?.animateMapStatus(u)

        // fill into map
        val icon = if (start) startDescriptor else destDescriptor
        val startMarker = MarkerOptions().icon(icon).position(latLng).zIndex(10)
        map()?.addOverlay(startMarker)
    }

    private fun map(): BaiduMap? {
        return guard_map?.map
    }

    @Subscribe
    fun onChooseAddr(event: GuardChooseEvent) {
        viewModel.setAddr(event)
        fillDescriptor(event.addr.latitude, event.addr.longitude, event.from)
    }


}