package com.lvwan.mobile110.fragment

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.RecyclerView
import android.view.View
import com.baidu.location.BDLocation
import com.common.annotation.LayoutId
import com.common.fragment.BindingFragment
import com.lvwan.mobile110.R
import com.lvwan.mobile110.activity.LocationActivity
import com.lvwan.mobile110.constant.CBundle
import com.lvwan.mobile110.databinding.FragmentHomeBinding
import com.lvwan.mobile110.stat.Action
import com.lvwan.mobile110.stat.ActionTracker
import com.lvwan.mobile110.viewmodel.HomeFragmentViewModel
import com.lvwan.util.LocationUtil
import com.lvwan.util.ScreenUtils
import com.lvwan.util.ViewUtils
import com.lvwan.util.startActivityForResultFromFragment
import kotlinx.android.synthetic.main.fragment_home.*
import org.jetbrains.anko.onClick

/**
 *
 * Created by wangdong on 17/3/14.
 */

@LayoutId(R.layout.fragment_home)
class HomeFragment : BindingFragment<HomeFragmentViewModel, FragmentHomeBinding>() {


    override fun onViewCreated(view: View?, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val titleH = resources.getDimension(R.dimen.common_titlebar_height).toInt()
        val bannerH = resources.getDimension(R.dimen.home_banner_height).toInt()

        // set titlebar height
        home_titlebar.layoutParams.height = ScreenUtils.statusbarheight + titleH
        home_titlebar_bg.alpha = 0.0f
        home_title.alpha = 0.0f
        home_titlebar.requestLayout()

        // set scroll listener
        home_recyclerView.addOnScrollListener(object: RecyclerView.OnScrollListener(){

            override fun onScrolled(recyclerView: RecyclerView?, dx: Int, dy: Int) {
                val deltaY = ViewUtils.getScollYDistance(recyclerView)
                val alpha = deltaY * 1.0f / bannerH
                home_titlebar_bg.alpha = alpha
                home_title.alpha = alpha
            }
        })

        // location
        requestLocation()

        home_loc.onClick {
            startActivityForResultFromFragment<LocationActivity>(LocationActivity.CITY_REQUEST_CODE)
        }

    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        registerLocation(true)
    }

    override fun onResume() {
        super.onResume()
        ActionTracker.trackAction(Action.LAUNCH)
        ActionTracker.upload()
    }

    override fun onDestroy() {
        super.onDestroy()
        registerLocation(false)
    }

    fun requestLocation() {
        LocationUtil.getInstance().requestLocation()
    }

    fun registerLocation(register: Boolean) {
        if (register) {
            LocationUtil.getInstance().registerLocationListener(locationListener)
        } else {
            LocationUtil.getInstance().unRegisterLocationListener(locationListener)
        }
    }

    val locationListener = { locationType: LocationUtil.LocationType, bdLocation: BDLocation? ->
        if (locationType == LocationUtil.LocationType.Succeed) {
            home_loc.text = bdLocation?.city
        }

    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == Activity.RESULT_OK) {
            if (requestCode == LocationActivity.CITY_REQUEST_CODE) {
                val city = data?.getStringExtra(CBundle.CITY)
                home_loc.setText(city)
            }
        }
    }

    companion object {
        val TAG = HomeFragment::class.java.simpleName
    }

}
