package com.lvwan.mobile110.fragment

import android.animation.Animator
import android.animation.AnimatorInflater
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.common.adapter.BaseViewHolder
import com.common.adapter.ClickAdapter
import com.common.adapter.ClickAdapterAssist
import com.common.net.HttpResponseListener
import com.lvwan.config.ErrorCode
import com.lvwan.mobile110.R
import com.lvwan.mobile110.constant.CBundle
import com.lvwan.mobile110.constant.CPreference
import com.lvwan.mobile110.entity.bean.MOTCityBean
import com.lvwan.mobile110.entity.bean.MOTCityBean.City
import com.lvwan.mobile110.entity.bean.MOTCityBean.City.County
import com.lvwan.mobile110.entity.bean.common.LWBean
import com.lvwan.mobile110.entity.event.MOTCountyEvent
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.mobile110.viewholder.MOTCityViewHolder
import com.lvwan.mobile110.viewholder.MOTCountyViewHolder
import com.lvwan.util.DialogUtils
import com.lvwan.util.LocationUtil
import com.lvwan.util.PreferenceHelper
import kotlinx.android.synthetic.main.fragment_mot_location.*
import org.greenrobot.eventbus.EventBus
import org.json.JSONObject
import java.util.*


/**
 * Created by wangdong on 16/8/2.
 *
 */
class MOTLocationFragment : BaseFragment(), HttpResponseListener<LWBean<MOTCityBean>> {


    private var province: MOTCityBean? = null

    private val cities: MutableList<City>

    private val counties: MutableList<County>

    private var selectedCity: City? = null

    init {
        cities = ArrayList<City>()
        counties = ArrayList<County>()
    }

    override fun bindView(inflater: LayoutInflater, container: ViewGroup): View {
        return inflater.inflate(R.layout.fragment_mot_location, container, false)
    }

    override fun onCreateAnimator(transit: Int, enter: Boolean, nextAnim: Int): Animator {
        if (enter) {
            return AnimatorInflater.loadAnimator(activity,
                    android.R.animator.fade_in)
        } else {
            return AnimatorInflater.loadAnimator(activity,
                    android.R.animator.fade_out)
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        // init city view
        mot_city_recyclerview.setLayoutManager(LinearLayoutManager(view.context))
        mot_city_recyclerview.setAdapter(ClickAdapter(cities, cityCallBack))

        // init country view
        mot_country_recyclerview.setLayoutManager(LinearLayoutManager(view.context))
        mot_country_recyclerview.setAdapter(ClickAdapter(counties, countryCallBack))

    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        // request province data
        val location = LocationUtil.getInstance().location
        var cityId: String? = null
        if (location != null) {
            cityId = location.cityCode
            PreferenceHelper.setString(CPreference.MOT_DETAIL_CITY_ID, cityId)
        }
        if (cityId != null) {
            RequestManager.instance().getCities(this, cityId)
        } else {
            showDiolog(getString(R.string.mot_gps_fail))
        }
    }

    private fun selectCity(position: Int) {
        // refresh city list
        for (i in cities.indices) {
            cities[i].selected = i == position
        }
        refreshCityData()

        selectedCity = cities[position]

        // refresh country list
        counties.clear()
        counties.addAll(selectedCity!!.counties)
        refreshCountryData()
    }

    // country adapter callback
    private val countryCallBack = object : ClickAdapterAssist {

        override fun recyclerViewItemOnClick(view: View, position: Int) {
            val provinceId = province!!.id
            val cityId = selectedCity!!.id
            val city = selectedCity!!.city
            val countryId = counties[position].id
            val county = counties[position].county
            EventBus.getDefault().post(MOTCountyEvent(provinceId, cityId, countryId, city, county))
        }

        override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>> {
            return MOTCountyViewHolder::class.java
        }
    }

    // city adapter callback
    private val cityCallBack = object : ClickAdapterAssist {

        override fun recyclerViewItemOnClick(view: View, position: Int) {
            selectCity(position)
        }

        override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>> {
            return MOTCityViewHolder::class.java
        }
    }

    private fun refreshCityData() {
        if (mot_city_recyclerview != null) {
            mot_city_recyclerview.getAdapter().notifyDataSetChanged()
        }
    }

    private fun refreshCountryData() {
        if (mot_country_recyclerview != null) {
            mot_country_recyclerview.getAdapter().notifyDataSetChanged()
        }
    }

    override fun onSuccess(lwBean: LWBean<MOTCityBean>) {
        if (lwBean.getError() == ErrorCode.ERR_MOT_CITY) {
            showDiolog(lwBean.getMessage())
        } else {

            province = lwBean.getData()
            cities.addAll(province!!.cities)
            var cityId: String? = null
            try {
                if (arguments != null) {
                    val json = arguments.getString(CBundle.MOT_DETAIL_CITY_JSON)
                    val `object` = JSONObject(json)
                    cityId = `object`.getString(CBundle.CITY_ID)

                } else {
                    val location = LocationUtil.getInstance().location
                    cityId = location.cityCode
                }

            } catch (e: Exception) {
            }

            // find current city
            for (i in cities.indices) {
                // get it
                val city = cities[i]
                if (city.id == cityId) {
                    selectCity(i)
                    EventBus.getDefault().post(MOTCountyEvent(city.counties[0].id, city.city, city.counties[0].county))
                    break
                }
            }

            refreshCityData()
            refreshCountryData()
        }
    }

    override fun onFail(e: Throwable) {

    }

    private fun showDiolog(message: String) {
        DialogUtils.showSimpleDialog2(activity, message) { _, _ -> activity.finish() }
    }
}
