package com.lvwan.mobile110.viewholder

import android.content.Context
import android.support.v4.view.PagerAdapter
import android.support.v4.view.ViewPager
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import com.bigkoo.convenientbanner.ConvenientBanner
import com.bigkoo.convenientbanner.holder.Holder
import com.common.adapter.BaseViewHolder
import com.common.annotation.LayoutId
import com.common.net.HttpResponseListener
import com.common.net.HttpSuccessListener
import com.common.utils.startActivity
import com.lvwan.config.ErrorCode
import com.lvwan.mobile110.R
import com.lvwan.mobile110.activity.*
import com.lvwan.mobile110.constant.CBundle
import com.lvwan.mobile110.entity.bean.BannerInfoBean
import com.lvwan.mobile110.entity.bean.CarListBean
import com.lvwan.mobile110.entity.bean.IconsInfoBean
import com.lvwan.mobile110.entity.bean.common.CarBean
import com.lvwan.mobile110.entity.bean.common.LWBean
import com.lvwan.mobile110.entity.event.LoginEvent
import com.lvwan.mobile110.entity.event.WZCXEvent
import com.lvwan.mobile110.helper.IconConfigHelper
import com.lvwan.mobile110.manager.UserManager
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.mobile110.net.VarUrls
import com.lvwan.mobile110.stat.Action
import com.lvwan.mobile110.stat.ActionTracker
import com.lvwan.mobile110.stat.Module
import com.lvwan.util.ImageUtils
import com.lvwan.util.J
import com.lvwan.util.PreferenceHelper
import com.lvwan.util.dp2px
import kotlinx.android.synthetic.main.viewholder_home_header.view.*
import kotlinx.android.synthetic.main.wzcx_card.view.*
import kotlinx.android.synthetic.main.wzcx_card_add.view.*
import org.greenrobot.eventbus.Subscribe
import org.jetbrains.anko.onClick
import rx.Observable
import java.util.*
import java.util.concurrent.TimeUnit


/**
 *
 * Created by wangdong on 17/3/3.
 */

@LayoutId(R.layout.viewholder_home_header)
class HomeHeaderViewHolder(view : View) : BaseViewHolder<Any>(view) {

    val cards = ArrayList<CarBean>()


    init {

        // request banner data
        requestBanner()

        // request icons data
        requestIcons()

        if (UserManager.isLogin()) {
            // request card data
            requestCards()
            // request renew cars
            requestRenew()
        }

        refreshViewPager()

    }

    fun refreshIcons(icons: List<IconsInfoBean.Icon>) {
        view.icon_layout.setIconsInfo(activity, icons, true)
    }


    fun refreshViewPager() {
        val viewContainter = ArrayList<View>()
        val inflater = LayoutInflater.from(context)

        // card item
        for (card in cards) {
            val view = inflater.inflate(R.layout.wzcx_card, view.home_viewpager, false)
            view.wzcx_card_number.text = card.plateNo
            view.wzcx_money.text = card.totalMoneyStr
            view.wzcx_point.text = card.totalPointStr.toString()
            view.wzcx_count.text = String.format(context.getString(R.string.wzcx_number), card.totalUntreated)
            if (card.totalUntreated == 0) {
                view.wzcx_card_no_record.visibility = View.VISIBLE
                view.wzcx_card_have_record.visibility = View.GONE
            } else {
                view.wzcx_card_no_record.visibility = View.GONE
                view.wzcx_card_have_record.visibility = View.VISIBLE
            }
            view.wzcx_card_trans.visibility = if (card.isOwner) View.GONE else View.VISIBLE
            view.onClick {
                if (card.isOwner) {
                    WZCXResultActivity.start(context, card.id, card.isQueryable)
                    ActionTracker.trackAction(Module.HOME, Action.HOME_WZCX)
                } else {
                    startActivity<CarTransActivity>(CBundle.ID to card.id, CBundle.CAR_NO to card.plateNo)
                }
            }
            viewContainter.add(view)
        }

        // add item
        val addView = inflater.inflate(R.layout.wzcx_card_add, view.home_viewpager, false)
        addView.wzcx_card_add.onClick {
            if (UserManager.isLogin()) {
                startActivity<WZCXAddActivity>(CBundle.CAR_BIND to true)
            } else {
                startActivity<LoginActivity>()
            }
        }
        viewContainter.add(addView)

        view.home_viewpager.adapter = WZCXPagerAdapter(viewContainter)
        view.home_viewpager.setOffscreenPageLimit(2)
        view.home_viewpager.setPageTransformer(true, ScrollOffsetTransformer())
    }

    fun requestBanner() {
        RequestManager.instance().getBannerInfo(object : HttpSuccessListener<BannerInfoBean> {
            override fun onSuccess(bannerInfoBean: BannerInfoBean) {
                if (bannerInfoBean.banners != null) {

                    // cast
                    @Suppress("UNCHECKED_CAST")
                    val banner: ConvenientBanner<BannerInfoBean.Item> =
                            view.home_banner as ConvenientBanner<BannerInfoBean.Item>

                    // init
                    banner.setPages({ ImageHolderView() },  bannerInfoBean.banners)
                            .setPageIndicator(intArrayOf(R.drawable.radius_gray2, R.drawable.radius_gray))
                            .setPageIndicatorAlign(ConvenientBanner.PageIndicatorAlign.CENTER_HORIZONTAL)

                    banner.setVisibility(View.VISIBLE)
                    banner.startTurning(5000)
                }
            }
        })
    }

    fun requestIcons() {
        RequestManager.instance().getIconsInfo(object : HttpResponseListener<IconsInfoBean> {
            override fun onSuccess(bean: IconsInfoBean) {
                VarUrls.parseIcons(bean)
                VarUrls.ICONS_HOME?.let { refreshIcons(it) }
                view.icon_layout.visibility = View.VISIBLE
            }

            override fun onFail(e: Throwable?) {
                refreshIcons(IconConfigHelper.getHomeIcons())
                view.icon_layout.visibility = View.VISIBLE
            }

        })
    }

    fun requestCards() {
        J.j("requestCards: start")
        RequestManager.instance().getCarList({
            bean: LWBean<CarListBean> ->
            if (bean.error == ErrorCode.ERR_CAR_TRANS) {
                J.j("requestCards: trans")
                Observable.just("").delay(1, TimeUnit.SECONDS).subscribe { requestCards() }
            } else if (!bean.existError()) {
                J.j("requestCards:" + bean.data.carList.size)
                cards.clear()
                cards.addAll(bean.data.carList)
                refreshViewPager()
            }
        })
    }


    fun requestRenew() {
        RequestManager.instance().renewCars { bean -> requestRenewResult(bean.sid) }
    }

    fun requestRenewResult(sid: String) {
        RequestManager.instance().getRenewCarsResult(sid, { lwBean ->
            if (lwBean.isNotReady) {
                // polling
                Observable.just(sid).delay(1, TimeUnit.SECONDS).subscribe { requestRenewResult(it) }
            } else if (!lwBean.existError()) {
                requestCards()
            }
        })
    }


    @Subscribe
    fun onRrefresh(event: WZCXEvent) {
        requestCards()
    }



    @Subscribe
    fun onLogin(event: LoginEvent) {
        cards.clear()
        if (event.event == LoginEvent.SUCCESS) {
            requestCards()
            if (view.icon_layout.clickFromHome) {
                if (PreferenceHelper.firstWZCXLoginFromHome()) {
                    PreferenceHelper.setWZCXLoginFromHome()
                    ActionTracker.trackAction(Module.HOME, Action.HOME_WZCX_LOGIN)
                }
            }
        } else if (event.event == LoginEvent.EXIT) {
            // hide cards
            refreshViewPager()
        }
    }


    inner class ImageHolderView : Holder<BannerInfoBean.Item> {

        private lateinit var imageView: ImageView

        override fun createView(context: Context): View? {
            imageView = ImageView(context)
            imageView.scaleType = ImageView.ScaleType.FIT_XY
            return imageView
        }

        override fun UpdateUI(context: Context, position: Int, data: BannerInfoBean.Item) {
            imageView.setImageResource(R.drawable.default_banner)
            ImageUtils.displayImage(data.image_url, imageView)
            imageView.setOnClickListener { WebActivity.start(context, data) }
        }

    }

    internal inner class WZCXPagerAdapter(private val views: ArrayList<View>) : PagerAdapter() {

        override fun getCount(): Int {
            return views.size
        }

        override fun isViewFromObject(arg0: View, arg1: Any): Boolean {
            return arg0 === arg1
        }

        override fun instantiateItem(container: ViewGroup, position: Int): Any {
            container.addView(views[position])
            return views[position]
        }

        override fun destroyItem(container: ViewGroup, position: Int, obj: Any) {
            container.removeView(views[position])
        }
    }

    inner class ScrollOffsetTransformer : ViewPager.PageTransformer {
        override fun transformPage(page: View, position: Float) {
            if (position > 0) {
                val ratio = maxOf(1 - position, 0.9f)
                page.setScaleX(ratio)
                page.setScaleY(ratio)
                page.alpha = maxOf(1 - position, 0.7f)
                page.setTranslationX(-50.dp2px() * position)
            }
        }
    }

    override fun bindView(data: Any?) { }

}