package com.lvwan.mobile110.viewmodel

import android.databinding.ObservableField
import android.support.v7.widget.RecyclerView
import android.view.View
import com.common.adapter.BaseViewHolder
import com.common.adapter.HeaderAdapter
import com.common.adapter.HeaderAdapterAssist
import com.common.viewmodel.ActivityViewModel
import com.common.viewmodel.IAdapter
import com.lvwan.mobile110.activity.GuardAddrActivity
import com.lvwan.mobile110.entity.bean.GuardSetAddrBean
import com.lvwan.mobile110.entity.event.GuardChooseEvent
import com.lvwan.mobile110.entity.realm.GuardAddrRealm
import com.lvwan.mobile110.viewholder.GuardAddrHeaderViewHolder
import com.lvwan.mobile110.viewholder.GuardAddrItemViewHolder
import com.vicpin.krealmextensions.querySorted
import com.vicpin.krealmextensions.save
import io.realm.Sort
import org.greenrobot.eventbus.EventBus

/**
 *
 * Created by wangdong on 17/3/3.
 */
class GuardAddrViewModel(activity: GuardAddrActivity?, tripStart: Boolean) :
        ActivityViewModel<GuardAddrActivity>(activity), HeaderAdapterAssist, IAdapter {

    // header data wrapper
    data class GuardAddrWrapper(var bean : GuardSetAddrBean?, var visible : Boolean)

    val headerData = GuardAddrWrapper(null, true)

    val adapter = ObservableField<RecyclerView.Adapter<*>>()

    // show data
    private val items = ArrayList<GuardAddrRealm>()
    // backup data
    private val itemsBackup = ArrayList<GuardAddrRealm>()

    var setMode = false

    var setHome = false

    var tripStart = false

    init {
        this.tripStart = tripStart
        val adapter = HeaderAdapter(items, headerData, this)
        this.adapter.set(adapter)
    }


    override fun recyclerViewItemOnClick(view: View?, position: Int) {
        if (setMode) {
            setMode = false
            val name = items.get(position).name
            requestAddrs(name)

            // restore
            itemsBackup.clear()
            itemsBackup.addAll(items)
            items.clear()

        } else {
            val addr = items.get(position)
            EventBus.getDefault().post(GuardChooseEvent(addr, tripStart))
            if (!tripStart) {
                addr.timestamp = System.currentTimeMillis()
                addr.save()
            }
            activity.finish()
        }

        getAdapter().notifyDataSetChanged()
    }

    override fun recyclerViewHeaderOnclick(view: View?) {

    }

    override fun getHeaderViewHolderType(): Class<out BaseViewHolder<*>> {
        return GuardAddrHeaderViewHolder::class.java
    }

    override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>> {
        return GuardAddrItemViewHolder::class.java
    }


    override fun getAdapter(): RecyclerView.Adapter<*> {
        return adapter.get()
    }

    fun requestAddrs(name : String?) {

    }

    fun refreshItems(items: MutableList<GuardAddrRealm>?, visible: Boolean) {
        this.items.clear()
        items?.let {
            this.items.addAll(it)
            headerData.visible = visible
        }

        if (items == null && !tripStart) {
            this.items.addAll(GuardAddrRealm().querySorted("timestamp", Sort.DESCENDING))
        }

        getAdapter().notifyDataSetChanged()
    }



    fun setAddr(home: Boolean) {
        setHome = home
        setMode = true
        itemsBackup.addAll(items)
        items.clear()
        getAdapter().notifyDataSetChanged()
    }


}