package com.lvwan.mobile110.viewmodel

import android.databinding.ObservableField
import android.support.v7.widget.RecyclerView
import android.text.TextUtils

import com.baidu.location.BDLocation
import com.common.adapter.BaseAdapter
import com.common.adapter.BaseAdapterAssist
import com.common.adapter.BaseViewHolder
import com.common.net.HttpSuccessListener
import com.common.viewmodel.FragmentViewModel
import com.common.viewmodel.IAdapter
import com.lvwan.mobile110.entity.bean.GuidePageBean
import com.lvwan.mobile110.fragment.GuideBusinessFragment
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.mobile110.viewholder.GuideBusinessInnerViewHolder
import com.lvwan.mobile110.viewholder.GuideBusinessTitleViewHolder
import com.lvwan.util.J
import com.lvwan.util.LocationUtil

import java.util.ArrayList

/**
 *
 * Created by wangdong on 16/12/5.
 */

class GuideBusinessViewModel(fragment: GuideBusinessFragment) : FragmentViewModel<GuideBusinessFragment>(fragment), HttpSuccessListener<List<GuidePageBean>>, IAdapter, BaseAdapterAssist {

    var adapter = ObservableField<RecyclerView.Adapter<*>>()

    private var page = 1

    private var index = 0

    private val items = ArrayList<Item>()

    private val beans = ArrayList<GuidePageBean>()


    init {
        val adapter = BaseAdapter(items, this)
        this.adapter.set(adapter)
        feedMore()
    }

    override fun getAdapter(): RecyclerView.Adapter<*> {
        return adapter.get()
    }

    override fun onSuccess(beans: List<GuidePageBean>) {
        ++page
        this.beans.addAll(beans)
        val items = bean2Item(beans)
        this.items.addAll(items)
        index += beans.size
        getAdapter().notifyDataSetChanged()

    }

    fun feedMore() {
        var area: String? = null
        try {
            val location = LocationUtil.getInstance().location
            area = location.district
            if (TextUtils.isEmpty(area)) {
                area = location.city
            }
        } catch (ignored: NullPointerException) {
        }

        RequestManager.instance().getGuidePage(page, area, this)
    }

    private fun bean2Item(beans: List<GuidePageBean>): List<Item> {
        val items = ArrayList<Item>()
        val max = 6
        for (j in beans.indices) {
            val bean = beans[j]
            val title = Item()
            val newIndex = this.index + j
            title.index = newIndex
            items.add(title)
            title.type = Item.TITLE
            title.title = bean.workTheme
            var i = 0
            while (i < bean.guides.size && i < max) {
                val item = Item()
                item.index = newIndex
                item.type = Item.INNER
                item.content1 = bean.guides[i].serviceName
                item.guideId1 = bean.guides[i].guideId
                if (i + 1 < bean.guides.size) {
                    item.content2 = bean.guides[i + 1].serviceName
                    item.guideId2 = bean.guides[i + 1].guideId
                }
                if (i + 2 < bean.guides.size && i + 2 >= max) {
                    item.more = true
                }
                items.add(item)
                i += 2
            }
        }
        return items
    }

    private fun fillItem(position: Int) {
        val index = items[position].index
        var titleIdx = 0

        run {
            var i = items.size - 1
            while (items.size >= 0) {
                if (items[i].index == index) {
                    if (items[i].type == Item.INNER) {
                        items.removeAt(i)
                    } else {
                        titleIdx = i
                        break
                    }
                }
                i--
            }
        }


        val bean = beans[index]

        var curIndex = 0
        var i = 0
        while (i < bean.guides.size) {
            val item = Item()
            item.index = index + this.index
            item.type = Item.INNER
            item.content1 = bean.guides[i].serviceName
            item.guideId1 = bean.guides[i].guideId
            if (i + 1 < bean.guides.size) {
                item.content2 = bean.guides[i + 1].serviceName
                item.guideId2 = bean.guides[i + 1].guideId
            }
            items.add(titleIdx + curIndex + 1, item)
            curIndex++
            i += 2
        }

    }


    fun showMore(position: Int) {
        fillItem(position)
        getAdapter().notifyDataSetChanged()
    }


    override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>>? {
        val type = items[position].type
        if (type == Item.TITLE) {
            return GuideBusinessTitleViewHolder::class.java
        } else if (type == Item.INNER) {
            return GuideBusinessInnerViewHolder::class.java
        }
        return null
    }


    class Item {

        var type: Int = 0

        var index: Int = 0

        var title: String? = null

        var content1: String? = null

        var content2: String? = null

        var guideId1: String? = null

        var guideId2: String? = null

        var more: Boolean = false

        companion object {

            val TITLE = 1

            val INNER = 2
        }
    }
}
