package com.lvwan.mobile110.viewmodel

import android.databinding.ObservableField
import android.view.View
import com.common.adapter.BaseViewHolder
import com.common.adapter.ClickAdapter
import com.common.adapter.ClickAdapterAssist
import com.common.viewmodel.FragmentViewModel
import com.lvwan.mobile110.entity.event.GuideQueryEvent
import com.lvwan.mobile110.fragment.GuideDepartmentFragment
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.mobile110.viewholder.GuideDepartmentViewHolder
import org.greenrobot.eventbus.EventBus
import java.util.*


/**
 *
 * Created by wangdong on 16/12/9.
 */

class GuideDepartmentViewModel(fragment: GuideDepartmentFragment, city: String?) : FragmentViewModel<GuideDepartmentFragment>(fragment) {

    var adapter = ObservableField<ClickAdapter<GuideDepartmentViewModel.Item>>()

    private val items = ArrayList<GuideDepartmentViewModel.Item>()

    init {
        request(city)
    }


    fun request(city: String?) {
        if (adapter.get() != null) {
            items.clear()
            adapter.get().notifyDataSetChanged()
        }

        RequestManager.instance().getGuideDepartment(city) { strings ->

            for (str in strings) {
                val item = GuideDepartmentViewModel.Item()
                item.item = str
                items.add(item)
            }

            if (strings.size > 0) {
                val item = GuideDepartmentViewModel.Item()
                item.item = "全部"
                items.add(0, item)
            }

            val adapter = ClickAdapter(items, object : ClickAdapterAssist {
                override fun recyclerViewItemOnClick(view: View, position: Int) {
                    // query event
                    val event = GuideQueryEvent(GuideQueryEvent.TYPE_DEPARTMENT)

                    if (position == 0) {
                        event.invalid = true
                    }

                    // update items
                    val current = items[position].selected
                    for (item in items) {
                        item.selected = false
                    }
                    if (!current) {
                        items[position].selected = true
                        event.department = items[position].item
                    }

                    // refresh ui
                    this@GuideDepartmentViewModel.adapter.get().notifyDataSetChanged()

                    // notify
                    EventBus.getDefault().post(event)

                }

                override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>> {
                    return GuideDepartmentViewHolder::class.java
                }
            })
            this.adapter.set(adapter)
        }
    }


    class Item {

        var item: String? = null

        var selected: Boolean = false

    }


}
