package com.lvwan.mobile110.viewmodel

import android.databinding.ObservableField
import android.view.View
import com.common.adapter.BaseViewHolder
import com.common.adapter.HeaderAdapterAssist
import com.common.adapter.LetterIndexAdapter
import com.common.viewmodel.FragmentViewModel
import com.lvwan.mobile110.constant.CBundle
import com.lvwan.mobile110.entity.event.GuideQueryEvent
import com.lvwan.mobile110.fragment.LocationFragment
import com.lvwan.mobile110.viewholder.GovGuideHeaderViewHolder
import com.lvwan.mobile110.viewholder.IndexAreaViewHolder
import com.lvwan.mobile110.widget.pickerview.JsonParserHandler
import com.lvwan.util.CharacterParser
import org.greenrobot.eventbus.EventBus
import java.util.*

/**
 *
 * Created by wangdong on 16/12/5.
 */

class GuideLocationViewModel(fragment: LocationFragment) : FragmentViewModel<LocationFragment>(fragment), HeaderAdapterAssist {

    var adapter2 = ObservableField<LetterIndexAdapter>()

    var location = ObservableField<String>()

    private val items = ArrayList<LetterIndexAdapter.Letter>()

    init {
        initialize()
    }

    fun initialize() {

        val names = initData() ?: return

        val characterParser = CharacterParser.getInstance()
        for (i in names.indices) {
            val pinyin = characterParser.getSelling(names[i])
            val sortString = pinyin.substring(0, 1).toUpperCase()
            if (sortString.matches("[A-Z]".toRegex())) {
                items.add(LetterIndexAdapter.Letter(names[i], sortString))
            }
        }

        Collections.sort(items, PinyinComparator())
        val adapter2 = LetterIndexAdapter(items, this)
        this.adapter2.set(adapter2)
    }

    private fun initData(): List<String>? {

        val names = ArrayList<String>()

        if (fragment.arguments != null && fragment.arguments.getBoolean(CBundle.ALL)) {
            val provinces = JsonParserHandler.parse(JsonParserHandler.CHINA)
            for (province in provinces) {
                if (province != null) {
                    names.addAll(province.cityNameList)
                    for (county in province.disNameList) {
                        names.addAll(county)
                    }
                }
            }
        } else {
            val province = JsonParserHandler.parse(JsonParserHandler.ZHEJIANG)[0]
            if (province != null) {
                names.addAll(province.cityNameList)
                for (county in province.disNameList) {
                    names.addAll(county)
                }
            }
        }

        return names
    }

    override fun recyclerViewHeaderOnclick(view: View) {

    }

    override fun getHeaderViewHolderType(): Class<out BaseViewHolder<*>> {
        return GovGuideHeaderViewHolder::class.java
    }

    override fun recyclerViewItemOnClick(view: View, position: Int) {
        val event = GuideQueryEvent(GuideQueryEvent.TYPE_AREA)

        // update items
        val current = items[position].selected
        for (item in items) {
            item.selected = false
        }
        if (!current) {
            items[position].selected = true
            event.area = items[position].name
        }

        // refresh ui
        adapter2.get().notifyDataSetChanged()

        // notify
        EventBus.getDefault().post(event)
    }

    override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>> {
        return IndexAreaViewHolder::class.java
    }

    fun getAdapter(): LetterIndexAdapter {
        return adapter2.get()
    }


    private inner class PinyinComparator : Comparator<LetterIndexAdapter.Letter> {

        override fun compare(o1: LetterIndexAdapter.Letter, o2: LetterIndexAdapter.Letter): Int {
            return o1.letter.compareTo(o2.letter)
        }

    }
}
