package com.lvwan.mobile110.viewmodel

import android.databinding.ObservableField
import android.view.View
import com.common.adapter.BaseViewHolder
import com.common.adapter.HeaderAdapter
import com.common.adapter.HeaderAdapterAssist
import com.common.viewmodel.FragmentViewModel
import com.lvwan.mobile110.R
import com.lvwan.mobile110.entity.event.GuideQueryEvent
import com.lvwan.mobile110.fragment.GuideMoreFragment
import com.lvwan.mobile110.viewholder.GuideMoreHeaderViewHolder
import com.lvwan.mobile110.viewholder.GuideMoreViewHolder
import org.greenrobot.eventbus.EventBus
import java.util.*

/**
 *
 * Created by wangdong on 16/12/5.
 */

class GuideMoreViewModel(fragment: GuideMoreFragment) : FragmentViewModel<GuideMoreFragment>(fragment), HeaderAdapterAssist {

    var adapter = ObservableField<HeaderAdapter<Item, Any>>()

    private val items = ArrayList<Item>()

    init {

        val themes = resources.getStringArray(R.array.guide_more_theme)

        for (theme in themes) {
            val item = Item()
            item.theme = theme
            this.items.add(item)
        }

        val adapter = HeaderAdapter(this.items, Any(), this)

        this.adapter.set(adapter)
    }


    override fun recyclerViewHeaderOnclick(view: View) {

    }

    override fun getHeaderViewHolderType(): Class<out BaseViewHolder<*>> {
        return GuideMoreHeaderViewHolder::class.java
    }

    override fun recyclerViewItemOnClick(view: View, position: Int) {

        // query event
        val event = GuideQueryEvent(GuideQueryEvent.TYPE_WORKTHEME)

        // update items
        val current = items[position].selected
        for (item in items) {
            item.selected = false
        }
        if (!current) {
            items[position].selected = true
            event.workTheme = items[position].theme
        }

        // refresh ui
        adapter.get().notifyDataSetChanged()

        // notify
        EventBus.getDefault().post(event)
    }

    override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>> {
        return GuideMoreViewHolder::class.java
    }

    class Item {

        var theme: String? = null

        var selected: Boolean = false

    }

}
