package com.lvwan.mobile110.viewmodel

import android.databinding.ObservableBoolean
import android.databinding.ObservableField
import android.support.v7.widget.RecyclerView
import android.view.View

import com.common.adapter.BaseViewHolder
import com.common.adapter.ClickAdapter
import com.common.adapter.ClickAdapterAssist
import com.common.viewmodel.FragmentViewModel
import com.common.viewmodel.IAdapter
import com.lvwan.mobile110.activity.GuideDetailActivity
import com.lvwan.mobile110.entity.bean.GuideQueryBean
import com.lvwan.mobile110.fragment.GuideResultFragment
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.mobile110.viewholder.GuideResultViewHolder

import java.util.ArrayList

/**
 *
 * Created by wangdong on 16/12/5.
 */

class GuideResultViewModel(fragment: GuideResultFragment) : FragmentViewModel<GuideResultFragment>(fragment), ClickAdapterAssist, IAdapter {

    var adapter = ObservableField<RecyclerView.Adapter<*>>()

    var visible = ObservableBoolean()

    private var area: String? = null
    private var department: String? = null
    private var target: String? = null
    private var workTheme: String? = null
    private var cond: String? = null

    private val items = ArrayList<GuideQueryBean>()

    init {

        val adapter = ClickAdapter(items, this)
        this.adapter.set(adapter)
    }


    fun setParams(area: String?, cond: String?, department: String?, target: String?, workTheme: String?) {
        this.area = area
        this.department = department
        this.target = target
        this.cond = cond
        this.workTheme = workTheme
    }

    fun requestList() {
        RequestManager.instance().getGuideQueryResult(area, cond, department, target, workTheme) { list ->
            items.clear()
            if (list != null && list.size > 0) {
                items.addAll(list)
                visible.set(true)
            } else {
                visible.set(false)
            }
            getAdapter().notifyDataSetChanged()
        }
    }


    override fun recyclerViewItemOnClick(view: View, position: Int) {
        GuideDetailActivity.start(activity, items[position].guideId)
    }

    override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>> {
        return GuideResultViewHolder::class.java
    }


    override fun getAdapter(): RecyclerView.Adapter<*> {
        return adapter.get()
    }
}
