package com.lvwan.mobile110.viewmodel

import android.databinding.ObservableField
import android.support.v7.widget.RecyclerView
import android.view.View
import com.common.adapter.BaseViewHolder
import com.common.adapter.HeaderAdapter
import com.common.adapter.HeaderAdapterAssist
import com.common.net.HttpSuccessListener
import com.common.viewmodel.FragmentViewModel
import com.common.viewmodel.IAdapter
import com.lvwan.mobile110.activity.WebActivity
import com.lvwan.mobile110.entity.bean.*
import com.lvwan.mobile110.fragment.HomeFragment
import com.lvwan.mobile110.helper.SchemeHelper
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.mobile110.viewholder.HomeHeaderViewHolder
import com.lvwan.mobile110.viewholder.HomeImageViewHolder
import com.lvwan.mobile110.viewholder.HomeNewsItemViewHolder

/**
 *
 * Created by wangdong on 17/3/14.
 */

class HomeFragmentViewModel(fragment: HomeFragment) :FragmentViewModel<HomeFragment>(fragment),
        HeaderAdapterAssist, IAdapter, HttpSuccessListener<List<BaseCardBean>> {


    val adapter = ObservableField<RecyclerView.Adapter<*>>()

    private val items = ArrayList<BaseCardBean>()

    init {
        val adapter = HeaderAdapter(items, Any(), this, activity)
        this.adapter.set(adapter)

        // request cards
        RequestManager.instance().getCardsInfo(this)
    }

    override fun onSuccess(list: List<BaseCardBean>?) {
        list?.let { items.addAll(it) }
    }

    override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>>? {
        val bean = items.get(position)
        if (bean is NewsCardBean) {
            // news viewholder
            return HomeNewsItemViewHolder::class.java
        } else if (bean is ImageCardBean) {
            // image viewholder
            return HomeImageViewHolder::class.java
        }

        // cannot run
        return null
    }

    override fun getHeaderViewHolderType(): Class<out BaseViewHolder<*>> {
        // header viewholder
        return HomeHeaderViewHolder::class.java
    }

    override fun recyclerViewItemOnClick(view: View?, position: Int) {
        val bean = items.get(position)
        if (bean is NewsCardBean) {
            if (bean is NativeCardBean) {
                // native click
                SchemeHelper.action(activity, bean.url)
            } else {
                // news click
                WebActivity.start(context, bean.url, bean.title, bean.share_desc, bean.share_icon)
            }
        }
    }

    override fun recyclerViewHeaderOnclick(view: View?) {
    }



    override fun getAdapter(): RecyclerView.Adapter<*> {
        return adapter.get()
    }

}
