package com.lvwan.mobile110.viewmodel

import android.databinding.ObservableBoolean
import android.databinding.ObservableField
import android.support.v7.widget.RecyclerView
import android.view.View
import com.common.adapter.BaseViewHolder
import com.common.adapter.HeaderAdapter
import com.common.adapter.HeaderAdapterAssist
import com.common.net.HttpResponseListener
import com.common.viewmodel.ActivityViewModel
import com.common.viewmodel.IAdapter
import com.lvwan.mobile110.activity.ImmigrationActivity
import com.lvwan.mobile110.activity.ImmigrationProgressActivity
import com.lvwan.mobile110.activity.WebActivity
import com.lvwan.mobile110.entity.bean.ImmigrationProgressBean
import com.lvwan.mobile110.entity.bean.SearchRecordsBean
import com.lvwan.mobile110.entity.bean.common.LWBean
import com.lvwan.mobile110.manager.UserManager
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.mobile110.net.VarUrls
import com.lvwan.mobile110.viewholder.ImmigrationHeaderViewHolder
import com.lvwan.mobile110.viewholder.ImmigrationItemViewHolder
import rx.Observable
import java.net.URLEncoder
import java.util.*
import java.util.concurrent.TimeUnit

/**
 *
 * Created by wangdong on 16/8/26.
 */
class ImmigrationViewModel(activity: ImmigrationActivity) : ActivityViewModel<ImmigrationActivity>(activity), HeaderAdapterAssist, IAdapter {

    var adapter = ObservableField<RecyclerView.Adapter<*>>()

    var loading = ObservableBoolean()

    var login = ObservableBoolean()

    private val items = ArrayList<SearchRecordsBean>()


    init {

        setAdapter(items)

        requestHistory()

    }

    fun requestHistory() {
        if (UserManager.isLogin()) {
            login.set(true)
            // RequestManager.instance().getImmigrationHistory(this::refresh);
        } else {
            login.set(false)
        }

        RequestManager.instance().getSearchRecords { beans ->
            items.clear()
            items.addAll(beans)
            getAdapter().notifyDataSetChanged()
        }

    }


    fun setLogin(login: Boolean) {
        this.login.set(login)
    }


    override fun getHeaderViewHolderType(): Class<out BaseViewHolder<*>> {
        return ImmigrationHeaderViewHolder::class.java
    }

    override fun getViewHolderType(position: Int): Class<out BaseViewHolder<*>> {
        return ImmigrationItemViewHolder::class.java
    }

    override fun recyclerViewHeaderOnclick(view: View) {

    }

    override fun recyclerViewItemOnClick(view: View, position: Int) {
        //        loading.set(true);
        //        int infoId = items.get(position).infoId;
        //        RequestManager.instance().requestHkmSid(sidBean -> getImmigrationDetail(sidBean.getSid()), infoId);
        val bean = items[position]
        var url = VarUrls.IMMI_RESULT?.url + "?type=%s&code=%s&infoId=%s&sessionId=%s&status=%s&name=%s&time=%s"
        url = String.format(url, bean.type, bean.code, bean.infoId, bean.sessionId, bean.status,
                URLEncoder.encode(bean.name, "UTF-8"), URLEncoder.encode(bean.time, "UTF-8"))
        WebActivity.start(activity, url)
    }

    private fun getImmigrationDetail(sid: String) {
        RequestManager.instance().getImmigrationResult(sid, object : HttpResponseListener<LWBean<ImmigrationProgressBean>> {
            override fun onFail(e: Throwable) {
                loading.set(false)
            }

            override fun onSuccess(lwBean: LWBean<ImmigrationProgressBean>) {
                if (lwBean.isNotReady) {
                    // polling
                    Observable.just(sid).delay(1, TimeUnit.SECONDS).subscribe({ this@ImmigrationViewModel.getImmigrationDetail(it) })

                } else if (lwBean.getData() != null) {
                    loading.set(false)
                    ImmigrationProgressActivity.start(activity, lwBean.getData())
                }
            }
        })
    }


    private fun setAdapter(beans: List<SearchRecordsBean>) {

        // new adapter
        val headerAdapter = HeaderAdapter(beans, Any(), this)

        // notify recyclerview
        adapter.set(headerAdapter)
    }

    private fun refresh(beans: List<SearchRecordsBean>) {
        items.clear()
        items.addAll(beans)
        adapter.get().notifyDataSetChanged()
    }


    override fun getAdapter(): RecyclerView.Adapter<*> {
        return adapter.get()
    }
}
