package com.lvwan.mobile110.viewmodel

import android.app.Activity
import android.content.Intent
import android.databinding.ObservableBoolean
import android.databinding.ObservableField
import android.databinding.ObservableInt
import com.common.net.HttpResponseListener
import com.common.viewmodel.ActivityViewModel
import com.lvwan.config.ErrorCode
import com.lvwan.mobile110.R
import com.lvwan.mobile110.activity.*
import com.lvwan.mobile110.constant.CBundle
import com.lvwan.mobile110.entity.bean.BindCarResultBean
import com.lvwan.mobile110.entity.bean.CarCityBean
import com.lvwan.mobile110.entity.bean.common.LWBean
import com.lvwan.mobile110.entity.bean.common.SidBean
import com.lvwan.mobile110.entity.event.WZCXEvent
import com.lvwan.mobile110.net.RequestManager
import com.lvwan.util.DialogUtils
import com.lvwan.util.LuaUtils
import com.lvwan.util.PreferenceHelper
import com.lvwan.util.startActivity
import org.greenrobot.eventbus.EventBus
import rx.Observable
import java.util.concurrent.TimeUnit

/**
 *
 * Created by wangdong on 17/5/11.
 */

class WZCXAddViewModel(activity: WZCXAddActivity, carBind: Boolean) : ActivityViewModel<WZCXAddActivity>(activity) {

    private val REQUEST_PRO = 1

    val engine = ObservableInt()

    val vin = ObservableInt()

    val cond = ObservableInt()

    val plateNo = ObservableField<String>()

    val engineTx = ObservableField<String>()

    val vinTx = ObservableField<String>()

    val condTx = ObservableField<String>()

    val loading = ObservableBoolean(false)

    val locked = ObservableBoolean(false)

    val enable = ObservableBoolean(false)

    val pro = ObservableField<String>()

    var cityBeans : MutableList<CarCityBean>? = null

    var bind = false

    var appealed = false

    var userLocked = false

    init {

        // set default pro
        pro.set(PreferenceHelper.getCarPro())

        // get input message
        requestConds()

        // get lock info
        requestUserStatus()

        // car no monitor
        plateNo.addOnPropertyChangedCallback(object: android.databinding.Observable.OnPropertyChangedCallback(){
            override fun onPropertyChanged(p0: android.databinding.Observable?, p1: Int) {
                checkEnable()
                cityBeans?.let {
                    if (plateNo.get().length == 1) {
                        comparePrefix(it)
                    }
                }
            }
        })

        val callback = object : android.databinding.Observable.OnPropertyChangedCallback() {
            override fun onPropertyChanged(observable: android.databinding.Observable, i: Int) {
                checkEnable()
            }
        }
        engineTx.addOnPropertyChangedCallback(callback)
        vinTx.addOnPropertyChangedCallback(callback)
        condTx.addOnPropertyChangedCallback(callback)

        bind = carBind

    }

    private fun checkEnable() {
        if (userLocked) {
            enable.set(false)
            return
        }

        if (plateNo.get().isNullOrEmpty()) {
            enable.set(false)
            return
        }

        if (engine.get() == -1) {
            if (engineTx.get().isNullOrEmpty()) {
                enable.set(false)
                return
            }
        }

        if (engine.get() > 0) {
            if (!engineTx.get().isNullOrEmpty() && engineTx.get().length != engine.get()) {
                enable.set(false)
                return
            }
        }

        if (vin.get() == -1) {
            if (vinTx.get().isNullOrEmpty()) {
                enable.set(false)
                return
            }
        }

        if (vin.get() > 0) {
            if (!vinTx.get().isNullOrEmpty() && vinTx.get().length != vin.get()) {
                enable.set(false)
                return
            }
        }

        if (cond.get() == -1) {
            val len = condTx.get()?.length
            if (len != 4 && len != 6) {
                enable.set(false)
                return
            }
        }

        enable.set(true)
    }

    private fun requestUserStatus() {
        RequestManager.instance().getCarStatus { bean ->
            locked.set(bean.isUserLock)
            appealed = bean.isAppealed
            userLocked = bean.isUserLock
            enable.set(false)
        }
    }


    fun onClickBind() {
          loading.set(true)
        RequestManager.instance().bindCar(getFullPlate(), condTx.get(), engineTx.get(), vinTx.get(), bind, object: HttpResponseListener<LWBean<SidBean>> {

            override fun onSuccess(lwBean: LWBean<SidBean>) {
                if (lwBean.error == ErrorCode.ERR_LUA_FAIL) {
                    LuaUtils.requestLuaKey({ onClickBind() })
                    return
                }

                if (lwBean.errorToast()) {
                    if (lwBean.error == ErrorCode.ERR_CAR_LOCKED) {
                        locked.set(true)
                    }
                    if (lwBean.error == ErrorCode.ERR_USER_LOCKED) {
                        locked.set(true)
                        userLocked = true
                        enable.set(false)
                    }
                    loading.set(false)
                    return
                }
                lwBean.data?.let { bindCar(it.sid) }

            }

            override fun onFail(e: Throwable) {
                loading.set(false)
            }

        })
    }

    fun getFullPlate(): String {
        return pro.get() + plateNo.get()
    }

    fun onClickDeblock() {
        if (appealed) {
            startActivity<DeblockingCheckingActivity>()
        } else {
            startActivity<DeblockingActivity>()
        }
    }

    private fun bindCar(sid: String) {
        RequestManager.instance().bindCarResult(sid, object: HttpResponseListener<LWBean<BindCarResultBean>> {

            override fun onSuccess(lwBean: LWBean<BindCarResultBean>) {
                    if (lwBean.isNotReady) {
                        // polling
                        Observable.just(sid).delay(3, TimeUnit.SECONDS).subscribe { bindCar(it) }

                    } else if (lwBean.data != null) {
                        if (bind) {
                            if (lwBean.data.isOwner) {
                                WZCXResultActivity.start(context, lwBean.data.carId, lwBean.data.isQueryable)
                                EventBus.getDefault().post(WZCXEvent())
                                activity.finish()
                            } else {
                                DialogUtils.showDialogWithCancel(
                                        activity, activity.getString(R.string.wzcx_bind_content),
                                        activity.getString(R.string.wzcx_bind_tip),
                                        { _, _ -> startActivity<CarTransActivity>(CBundle.ID to lwBean.data.carId, CBundle.CAR_NO to getFullPlate()) })
                            }
                        } else {
                            WZCXResultActivity.start(context, lwBean.data.carId, lwBean.data.isQueryable)
                            activity.finish()
                        }
                        loading.set(false)
                    } else {
                        lwBean.errorToast()
                        loading.set(false)
                    }
                }

                override fun onFail(e: Throwable?) {
                    loading.set(false)
                }

            })

    }

    fun onClickPro() {
        MoveCarNumProActivity.startForPro(activity, REQUEST_PRO, pro.get())
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_PRO && resultCode == Activity.RESULT_OK) {
            val pro = data?.getStringExtra(MoveCarNumProActivity.DATA_KEY)
            PreferenceHelper.setCarPro(pro)
            this.pro.set(pro)
            requestConds()
        }
    }

    fun requestConds() {

        if (cityBeans != null) {
            comparePrefix(cityBeans!!)
            return
        }

        RequestManager.instance().getCarConds(object: HttpResponseListener<LWBean<List<CarCityBean>>>{
            override fun onSuccess(lwBean: LWBean<List<CarCityBean>>) {
                lwBean.data?.let {
                    cityBeans = ArrayList<CarCityBean>()
                    cityBeans!!.addAll(lwBean.data)
                    comparePrefix(cityBeans!!)
                }
            }

            override fun onFail(e: Throwable) {
            }
        })
    }

    fun comparePrefix(beans: MutableList<CarCityBean>): Boolean {
        if (plateNo.get().isNullOrEmpty()) {
            return comparePrefixInner(pro.get(), beans)
        } else {
            val prefix = pro.get() + plateNo.get().subSequence(0, 1)
            return comparePrefixInner(prefix, beans)
        }
    }

    fun comparePrefixInner(prefix: String, beans: MutableList<CarCityBean>): Boolean {
        for (bean in beans) {
            if (prefix == bean.platePrefix || prefix.contains(bean.platePrefix)) {
                engine.set(bean.engineNoLength)
                vin.set(bean.vinLength)
                cond.set(bean.conditionLength)
                return true
            }
        }
        engine.set(0)
        vin.set(6)
        cond.set(0)
        return false
    }

}
